package main

import (
	"context"
	"fmt"
	"log"
	"net/http"
	"net/url"
	"os"

	handling "github.com/coreos/discovery.etcd.io/http"

	"github.com/coreos/go-systemd/activation"
	"github.com/spf13/pflag"
	"github.com/spf13/viper"
        "github.com/go-ini/ini"
)

func fail(err string) {
	log.Print(err)
	pflag.PrintDefaults()
	os.Exit(2) // default go flag error code
}

func mustHostOnlyURL(givenUrl string) string {
	u, err := url.Parse(givenUrl)

	if err != nil {
		fail(fmt.Sprintf("Invalid url given: %v", err))
	}

	if u.RawQuery != "" {
		fail(fmt.Sprintf("Expected url without query (?%v)", u.RawQuery))
	}

	if u.Fragment != "" {
		fail(fmt.Sprintf("Expected url without fragment (%v)", u.Fragment))
	}

	if u.Host == "" {
		fail(fmt.Sprint("Expected hostname (none given)"))
	}

	return u.Scheme + "://" + u.Host
}

func init() {
	var tls bool
	var insecure bool
	viper.SetEnvPrefix("disc")
	viper.AutomaticEnv()

	cfg, err := ini.Load("/etc/etcd-discovery/etcd-discovery.conf")
	if err != nil {
		fmt.Printf("Fail to read file: %v", err)
		os.Exit(1)
	}

	pflag.StringP("etcd", "e", cfg.Section("etcd").Key("endpoint_location").String(), "etcd endpoint location")
	pflag.StringP("host", "h", cfg.Section("server").Key("prefix_url").String(), "discovery url prefix")
	pflag.StringP("addr", "a", cfg.Section("server").Key("web_service_address").String(), "web service address")
	pflag.BoolVarP(&insecure, "insecure", "i", false, "insecure etcd endpoint")
	pflag.BoolVarP(&tls, "tls", "t", false, "tls etcd endpoint")

	viper.BindPFlag("etcd", pflag.Lookup("etcd"))
	viper.BindPFlag("host", pflag.Lookup("host"))
	viper.BindPFlag("addr", pflag.Lookup("addr"))
	viper.BindPFlag("insecure", pflag.Lookup("insecure"))
	viper.BindPFlag("tls", pflag.Lookup("tls"))

	pflag.Parse()
}

func main() {
	log.SetFlags(0)
	etcdHost := viper.GetString("etcd")
	discHost := viper.GetString("host")
	webAddr := viper.GetString("addr")
	insecure := viper.GetBool("insecure")
	tls := viper.GetBool("tls")

	handling.Setup(context.Background(), etcdHost, discHost, insecure, tls)

	log.Printf("discovery server started with etcd %q and host %q", etcdHost, discHost)
	log.Printf("discovery serving on %s", webAddr)
	err := http.ListenAndServe(webAddr, nil)
	if err != nil {
		panic(err)
	}

	listeners, err := activation.Listeners()
	if err != nil {
		panic(err)
	}

	if len(listeners) != 1 {
		panic("Unexpected number of socket activation fds")
	}

	http.Serve(listeners[0], nil)
}
